#include <Wire.h>

//#define DEBUG_
#define USE_FLOAT  //If this is defined the Resolution is boosted a bit

//Define ADC Samples
#define ADC_SAMPLES 4

//Warning Levels
float Warning_Used_Cap = 1750.0f;
float Warning_Current  = 70.0f;

//Application Data initial Array
byte SpektrumData[] = {0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
int sensorPin = A0;    // select the input pin for the Current Sensor
int led = 13;

//Time values for Used Capacity calculation 
unsigned long current_time = 0;
unsigned long last_time = 0;
float time_elapsed = 0.0f;

//Calculate this scalse factor here once to save cycles
float secound_scale = 1000.0f / 3600.0f;

//Sensor characteristics have to be determind for every sensor: use multiple data and calculate the values with the minimum quadratic diference methode
//I = I_a*ADC+I_b
float I_a = 0.1323594f;
float I_b = -16.13235294f;

// variable to store the value coming from the sensor
int sensorValue = 0;  

//Internal Values
float Current = 0.0f;
float Capacity_Used = 0.0f;
//Spektrum Values
word s_Current = 0;
word s_Cap = 0;


void setup() {
  // put your setup code here, to run once:
  // initialize the digital pin as an output.
  pinMode(led, OUTPUT);
    
  //Listen to I2C Address 0x0A
  Wire.begin(0x0A); 
  //Set Event on Addressed
  Wire.onRequest(requestEvent);
  
  //Get the Current time
  last_time = millis();
  
  //Debug
  #ifdef DEBUG_
  Serial.begin(19200);
  #endif
  
  digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
}

void loop() {
  // put your main code here, to run repeatedly: 
  // read the value from the sensor:
  sensorValue=0;
  //Get Samples from ADC
  for(int i=0;i<ADC_SAMPLES;i++)
  {
    sensorValue += analogRead(sensorPin);
  }
  //Get the Average
  #ifdef USE_FLOAT
    //Try to get a bit more resolution (This is Physicaly not absolotly correct [acording to valid numbers] but often be used in measurement applications to boost resolution a bit)
    Current = (float)sensorValue / (float)ADC_SAMPLES;
  #else
    //Faster thand divide by Samples
    //This only works for 4 Samples!!!! Change for other Samples analog
    sensorValue = sensorValue >> 2;
  #endif
  
  //Get the Current time
  current_time = millis();
  
  //Calculate the Current in [A]
  #ifdef USE_FLOAT
    //If used Floating Point
    Current = I_a * Current + I_b;
  #else
    //If normal Integer sensorValue is used
    Current = I_a * (float)sensorValue + I_b;
  #endif
  
  //Calculate the used Capacity in [mAh]
  //Calculate elapsed Time in sec first
  time_elapsed = (float)(current_time - last_time)/1000.0f;
  //Now the used capacity
  Capacity_Used += Current * secound_scale * time_elapsed;
  
  //Check and Flag Warnings
  if(Capacity_Used > Warning_Used_Cap)
  {
      bitSet(SpektrumData[15],2);
  }
  if(Current > Warning_Current)
  {
     bitSet(SpektrumData[15],0);
  }
  else
  {
    bitClear(SpektrumData[15],0); 
  }
  
  #ifdef DEBUG_
  #ifdef USE_FLOAT
  sensorValue = sensorValue >> 2;
  #endif
  //Calculate Spektrum Values
  s_Current = (word)(Current * 100.0f);
  s_Cap = (word)Capacity_Used;
  //Transmitt Data
  Serial.print("ADC: ");
  Serial.print(sensorValue);
  Serial.print(" I: ");
  Serial.print(Current);
  Serial.print(" Time: ");
  Serial.print(time_elapsed);
  Serial.print(" Cap: ");
  Serial.println(Capacity_Used);
  Serial.print(" Spektrum I: ");
  Serial.print(s_Current);
  Serial.print(" Cap: ");
  Serial.println(s_Cap); 
  #endif
 
  //Now save last time the loop has executed
  last_time = current_time;
  
  //Now wait a specific time to do loop again 
  delay(80);
}

void requestEvent()
{
  //This is a debug signal
  //digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
  
  //Calculate Spektrum Values
  s_Current = (word)(Current * 100.0f);
  s_Cap = (word)Capacity_Used;
  
  //Insert Data into SpektrumData
  //Volt 1 will be Current nad Cap 1 will be used Capacity
  SpektrumData[2] = highByte(s_Current);
  SpektrumData[3] = lowByte(s_Current);
  SpektrumData[6] = highByte(s_Cap);
  SpektrumData[7] = lowByte(s_Cap);
  
  //Transmit SpektrumData
  Wire.write(SpektrumData, 16); 
}
